@@ -27,7 +27,7 @@ class Client:
         - send examples
         - save weights and SavedModel to Picsell.ia server."""
 
-    def __init__(self, api_token, host="http://localhost:8000/sdk/"):
+    def __init__(self, api_token, host="http://localhost:8000/sdk/", interactive=True):
         """ Creates and initializes a Picsell.ia Client.
         Args:
             api_token (str): api_token key, given on the platform.
@@ -78,10 +78,12 @@ class Client:
         self.exported_model_dir = None
 
         # Experiment Variables
+        self.exp_id = None
         self.exp_name = None
         self.exp_description = None 
         self.exp_status = None 
         self.exp_parameters = None
+        self.line_nb = 0
         self.training_id = None
         self.annotation_type = None
         self.dict_annotations = {}
@@ -99,7 +101,7 @@ class Client:
         self.urls = urls(self.host, self.auth)
         self.dataset = self.Dataset(self.host, self.auth)
         
-
+        self.interactive = interactive
         print(f"Welcome {self.username}, glad to have you back")
 
 
@@ -143,9 +145,29 @@ class Client:
                     raise NotImplementedError(f"Found a non supported filetype {filename.split('.')[-1]} in your png_dir, \
                                     supported filetype are : {self.supported_img_types}, please move {filename} elsewhere")
 
-    def fetch_experiment_parameters(self, experiment_id):
+    def init_experiment(self, name):
+        assert isinstance(name, str), f"model name must be string, got {type(name)}"
+        assert name!="", "Experiment name's length can't be 0"
+        assert self.project_token!=None, "Please create or checkout project first."
         to_send = {
             "project_token": self.project_token,
+            "name": name,
+        }
+        try:
+            r = requests.post(self.host + 'init_experiment', data=json.dumps(to_send), headers=self.auth)
+        except Exception:
+            raise exceptions.NetworkError("Server is not responding, please check your host or Picsell.ia server status on twitter")
+        if r.status_code!=200:
+            raise exceptions.AuthenticationError(r.text)
+        response = r.json()
+        if response["name"]!=name:
+            print("{} experiment name was already taken, we have renamed it to {}".format(name,response["name"]))
+        self.exp_id = response["experiment_id"]
+        print("Experiment created successfully.")
+        return self.exp_id
+
+    def fetch_experiment_parameters(self, experiment_id):
+        to_send = {
             "experiment_id": experiment_id,
         }
         try:
@@ -155,6 +177,7 @@ class Client:
         response = r.json()
         if not response["exp_available"]:
             raise exceptions.ResourceNotFoundError("Experiment Not Found, please check your experiment_id")
+        self.exp_id = experiment_id
         self.exp_name = response["name"]
         self.exp_description = response["description"]
         self.exp_status = response["status"]
@@ -162,10 +185,42 @@ class Client:
         self.project_token = response["project_token"]
         if self.exp_status == 'waiting':
             print("Experiment {} found, ready for training. \n".format(self.exp_name))
+            self.update_experiment_status("launched")
         else:
-            print("Experiment {} found with status {} \n".format(self.exp_name, self.exp_status))
-        return self.project_token, self.exp_parameters
+            print("Experiment {} found with status '{}'\n".format(self.exp_name, self.exp_status))
         
+        return self.project_token, self.exp_parameters
+    
+    def update_experiment_status(self, status):
+        assert status!="", "Experiment status's length can't be 0"
+        assert self.exp_id!=None, "Please create or checkout experiment first."
+        to_send = {
+            "experiment_id": self.exp_id,
+            "status": status,
+        }
+        try:
+            r = requests.post(self.host + 'update_experiment_status', data=json.dumps(to_send), headers=self.auth)
+        except Exception:
+            raise exceptions.NetworkError("Server is not responding, please check your host or Picsell.ia server status on twitter")
+        if r.status_code!=200:
+            raise exceptions.AuthenticationError(r.text)
+        
+    def send_experiment_logging(self, log, part, final=False, special=False):
+        assert self.exp_id!=None, "Please create or checkout experiment first."
+        to_send = {
+            "experiment_id": self.exp_id,
+            "line_nb": self.line_nb,
+            "log": log,
+            "final": final,
+            "part": part,
+            "special": special
+        }
+        self.line_nb +=1
+        try:
+            r = requests.post(self.host + 'send_experiment_logging', data=json.dumps(to_send), headers=self.auth)
+        except Exception:
+            raise exceptions.NetworkError("Server is not responding, please check your host or Picsell.ia server status on twitter")
+
     def create_network(self, network_name, orphan=False):
         """ Initialise the model instance on Picsell.ia server.
             If the model name exists on the server for this project, you will create a new version of your training.
@@ -479,26 +534,31 @@ class Client:
             for _id in training_ids:
                 path = os.path.join(self.project_name, self.network_name, str(_id), "checkpoint")
                 if utils.is_checkpoint(path, self.project_type):
-
-                    while (a not in ["y", "yes", "n", "no"]):
-                        a = input(f"Found checkpoint for training {_id}, do you want to use this checkpoint ? [y/n] ")
-                    if a.lower() == 'y' or a.lower() == 'yes':
-                        p = os.path.join(self.project_name, self.network_name, str(self.training_id), "checkpoint")
-                        print(f"Your next training will use checkpoint: {p}")
-                        return path
-                    else:
-                        continue
-                else:
-                    path_deep = os.path.join(self.project_name, self.network_name, str(_id), "checkpoint", "origin")
-                    if utils.is_checkpoint(path_deep, self.project_type):
+                    if self.interactive:
                         while (a not in ["y", "yes", "n", "no"]):
-                            a = input(f"Found origin checkpoint from training {_id}, do you want to use this checkpoint ? [y/n] ")
+                            a = input(f"Found checkpoint for training {_id}, do you want to use this checkpoint ? [y/n] ")
                         if a.lower() == 'y' or a.lower() == 'yes':
                             p = os.path.join(self.project_name, self.network_name, str(self.training_id), "checkpoint")
                             print(f"Your next training will use checkpoint: {p}")
-                            return path_deep
+                            return path
                         else:
                             continue
+                    else:
+                        return path
+                else:
+                    path_deep = os.path.join(self.project_name, self.network_name, str(_id), "checkpoint", "origin")
+                    if utils.is_checkpoint(path_deep, self.project_type):
+                        if self.interactive:
+                            while (a not in ["y", "yes", "n", "no"]):
+                                a = input(f"Found origin checkpoint from training {_id}, do you want to use this checkpoint ? [y/n] ")
+                            if a.lower() == 'y' or a.lower() == 'yes':
+                                p = os.path.join(self.project_name, self.network_name, str(self.training_id), "checkpoint")
+                                print(f"Your next training will use checkpoint: {p}")
+                                return path_deep
+                            else:
+                                continue
+                        else:
+                            return path_deep
                     else:
                         continue
         else:
@@ -553,9 +613,10 @@ class Client:
 
         url_data = self.urls._get_presigned_url('get', self.checkpoint_data, bucket_model=True)
         checkpoint_file = os.path.join(path_to_origin, self.checkpoint_data.split('/')[-1])
-
+        
         with open(checkpoint_file, 'wb') as handler:
             print(f"Downloading {self.checkpoint_data}")
+            print('-----')    
             response = requests.get(url_data, stream=True)
             total_length = response.headers.get('content-length')
             if total_length is None:  # no content length header
@@ -563,14 +624,20 @@ class Client:
                 self.checkpoint_data = None
             else:
                 dl = 0
+                count = 0
                 total_length = int(total_length)
                 for data in response.iter_content(chunk_size=4096):
                     dl += len(data)
                     handler.write(data)
                     done = int(50 * dl / total_length)
-                    sys.stdout.write(f"\r[{'=' * done}{' ' * (50 - done)}]")
-                    sys.stdout.flush()
-            print()
+                    if self.interactive:
+                        sys.stdout.write(f"\r[{'=' * done}{' ' * (50 - done)}]")
+                        sys.stdout.flush()
+                    else:
+                        if count%500==0:
+                            print('['+'='* done+' ' * (50 - done)+']')
+                    count += 1
+        print('--*--')
         return path_to_origin
 
     def dl_train_test_split(self, prop):
@@ -698,13 +765,15 @@ class Client:
         for info in self.dict_annotations["images"]:
             lst.append(info["external_picture_url"])
         t = len(set(lst))
-        nb_threads = 50
+        print('-----')
+        nb_threads = 20
         infos_split = list(mlt.chunks(self.dict_annotations["images"], nb_threads))
         counter = Value('i', 0)
-        p = Pool(nb_threads, initializer=mlt.pool_init, initargs=(t, self.png_dir, counter,))
-        # p = ThreadPool(nb_threads, initializer=mlt.pool_init(t, self.png_dir), initargs=())
+        p = Pool(nb_threads, initializer=mlt.pool_init, 
+            initargs=(t, self.png_dir, counter,self.interactive,))
         p.map(mlt.dl_list, infos_split)
-        print("images downloaded")
+        print('--*--')
+        print("Images downloaded")
         #print(f"\n{cnt} images have been downloaded")
 
     def train_test_split(self, prop=0.8):
